################################################################################
## download and transform the data 
################################################################################


rm(list=ls()) 
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))

listofpackages <- c("tidyverse","ellipse","reshape2","xts","xlsx","readxl",
                    "quantmod")
#installation of "xlsx" requires Java 

for (j in listofpackages){
  if(sum(installed.packages()[, 1] == j) == 0) {
    install.packages(j)
  }
  library(j, character.only = T)
}

#
#Download DJI Data from yahoo
# begin from 1992-02-01 as the index is available only from then on


tickers <- c('AXP','AMGN','AAPL','BA','CAT','CSCO','CVX','GS','HD','HON',
             'IBM','INTC','JNJ','KO','JPM','MCD','MMM','MRK','MSFT','NKE',
             'PG','TRV','UNH','CRM','VZ','V','WBA','WMT','DIS','DOW','^DJI')

getSymbols.yahoo(tickers,
                 env = globalenv(),
                 index.class ='Date',
                 from = "1992-02-01",
                 to = "2023-10-31",
                 periodicity ="monthly")
# drop the stocks not available for the whole sample: CRM,DOW,GS,V,
stocks = merge(AXP[,6],AMGN[,6],AAPL[,6],BA[,6],CAT[,6],CSCO[,6],CVX[,6],HD[,6],HON[,6],
               IBM[,6],INTC[,6],JNJ[,6],KO[,6],JPM[,6],MCD[,6],MMM[,6],MRK[,6],MSFT[,6],NKE[,6],
               PG[,6],TRV[,6],UNH[,6],VZ[,6],WBA[,6],WMT[,6],DIS[,6],DJI[,6])

colnames(stocks) <- c('AXP','AMGN','AAPL','BA','CAT','CSCO','CVX','HD','HON',
                      'IBM','INTC','JNJ','KO','JPM','MCD','MMM','MRK','MSFT','NKE',
                      'PG','TRV','UNH','VZ','WBA','WMT','DIS','DJI')

#compute returns from stock prices
stock_names <- c('AXP','AMGN','AAPL','BA','CAT','CSCO','CVX','HD','HON',
                 'IBM','INTC','JNJ','KO','JPM','MCD','MMM','MRK','MSFT','NKE',
                 'PG','TRV','UNH','VZ','WBA','WMT','DIS','DJI')

t1<-nrow(stocks)

for (name in stock_names) {
  return_col_name <- paste0(name, "_ret")
  temporary_column  <- array(data = NA, dim = t1)
  
  stocks <- merge.xts(stocks, temporary_column) # add last column
  colnames(stocks)[ncol(stocks)] = return_col_name # rename it  
  
  for (i in 2:nrow(stocks)) {
    stocks[i, return_col_name] <- (stocks[i, name][[1]] - stocks[i - 1, name][[1]]) / stocks[i - 1, name][[1]]
  }
}

#write.xlsx(as.data.frame(stocks), "2023_monthly_stocks.xlsx", row.names = TRUE)
rm(list = c('AXP','AMGN','AAPL','BA','CAT','CSCO','CVX','GS','HD','HON',
            'IBM','INTC','JNJ','KO','JPM','MCD','MMM','MRK','MSFT','NKE',
            'PG','TRV','UNH','CRM','VZ','V','WBA','WMT','DIS','DOW','DJI'))
stocks_ret<-stocks[,28:ncol(stocks)]
#
# Download FF5 Data
#

url <- "https://mba.tuck.dartmouth.edu/pages/faculty/ken.french/ftp/F-F_Research_Data_5_Factors_2x3_CSV.zip"

# Specify the destination folder to save the downloaded file
download_folder <- getwd()

# Define the file path for the downloaded ZIP file
zip_file <- file.path(download_folder, "F-F_Research_Data_5_Factors_2x3_CSV.zip")

# Download the ZIP file
download.file(url, zip_file)

# Unzip the file
unzip(zip_file, exdir = download_folder)

# Identify the CSV file within the extracted folder
csv_file <- list.files(download_folder, pattern = "\\.csv$", full.names = TRUE)

# Read the CSV file into a data frame
ff_5_factors <- read.csv(csv_file, skip=2)
names(ff_5_factors)[1] = 'Date'
ff_5_factors[, -1] <- lapply(ff_5_factors[, -1], function(x) as.numeric(as.character(x)))
# Create an xts object with the desired date format
dates <-seq(as.Date("1963-07-01"),length=nrow(ff_5_factors), by="months")

xts_data <- xts(ff_5_factors[, -1], order.by = dates)
#rescale data 
xts_data <-0.01*xts_data
xts_data$Mkt<-array(data = NA, dim = nrow(xts_data))
xts_data$Mkt<-xts_data$Mkt.RF+xts_data$RF
start_date <- as.Date("1992-02-01")
end_date <- as.Date("2023-10-01")

# Create a new xts object with the specified date range
ff5 <- window(xts_data, start = start_date, end = end_date)

save(ff5,file="ff5.rda")
save(stocks_ret,file="stocks_ret.rda")

