# Answers to Q2 

rm(list=ls()) 
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))

listofpackages <- c("tidyverse","ellipse","reshape2","xts","xlsx","readxl",
                    "quantmod")
#installation of "xlsx" requires Java 

for (j in listofpackages){
  if(sum(installed.packages()[, 1] == j) == 0) {
    install.packages(j)
  }
  library(j, character.only = T)
}

#load data 
load('ff5.rda')
load('stocks_ret.rda')

#
# 1. Comparative performance analysis 
#
ff5$ff5_ret <- rowMeans(ff5[,1:5])
stocks_ret$DJ26_ret<- rowMeans(stocks_ret[,1:26])
data_perf<- cbind(ff5$ff5_ret,ff5$Mkt, stocks_ret$DJ26_ret,stocks_ret$DJI_ret)
colnames(data_perf)[colnames(data_perf) == "Mkt"] <- "Mkt_ret"

names <- colnames(data_perf)

t1<-nrow(data_perf)

for (name in names) {
  return_col_name <- paste0("Port_",name)
  temporary_column  <- array(data = 1, dim = t1)
  data_perf <- merge.xts(data_perf, temporary_column) # add last column
  colnames(data_perf)[ncol(data_perf)] = return_col_name # rename it
  
  for (i in 2:nrow(data_perf)) {
    data_perf[i, return_col_name][[1]] <- data_perf[i-1, return_col_name][[1]]* (1+data_perf[i, name][[1]])
  }
}

# time series Plot of the four Portfolios
plot <- ggplot(data= data_perf["1992-02-01/2005-06-01"], aes(x = index(data_perf["1992-02-01/2005-06-01"]))) +
  geom_line(aes(y = Port_ff5_ret, color = "Port_ff5_ret"), linewidth = 1) +
  geom_line(aes(y = Port_Mkt_ret, color = "Port_Mkt_ret"), linewidth = 1) +
    geom_line(aes(y = Port_DJ26_ret, color = "Port_DJ26_ret"), linewidth = 1) +  
  geom_line(aes(y = Port_DJI_ret, color = "Port_DJI_ret"), linewidth = 1) +
  labs(title = "Cumulative Returns of 1$ dollar invested",
       x = "Time", y = "Cum.Monthly Returns") +
  scale_color_manual(values = c("Port_ff5_ret" = "red","Port_Mkt_ret" = "green", "Port_DJ26_ret" = "lightblue","Port_DJI_ret" = "darkblue")) +
  theme_minimal() +
  theme(axis.line = element_line(color = "black"))

print(plot)


#efficient frontier
install.packages(c("cluster","mvoutlier","pastecs","fPortfolio"),repos="http://cran.r-project.org")
# load required packages
library(cluster)
library(mvoutlier)
library(pastecs)
library(fPortfolio)
#create excess returns

t1<-nrow(stocks_ret)
names <- colnames(stocks_ret)
for (name in names) {
  return_col_name <- paste0(name,"_ex")
  temporary_column  <- array(data = 1, dim = t1)
  stocks_ret <- merge.xts(stocks_ret, temporary_column) # add last column
  colnames(stocks_ret)[ncol(stocks_ret)] = return_col_name # rename it
}

  for (i in 29:56) {
    stocks_ret[, i] <- stocks_ret[, i-28]-ff5[, 6]
  }
stocks_exret<- (stocks_ret["1992-03-01/2005-06-01",29:54])

data01ts <- as.timeSeries(stocks_ret["1992-03-01/2005-06-01",1:26])

Data<-data01ts
tgSpec <- portfolioSpec()
setRiskFreeRate(tgSpec) <- 0
constraints <- "longOnly"
tgPortfolio <- tangencyPortfolio(
  data = data01ts,
  spec = tgSpec, constraints = constraints) 
print(tgPortfolio)

efficient_frontier <- portfolioFrontier(data = data01ts, spec = tgSpec, constraints = constraints)
print(efficient_frontier)
# Efficient Frontier plot 
setNFrontierPoints(tgSpec) <- 25
efficient_frontier <- portfolioFrontier(data = data01ts, spec = tgSpec, constraints = constraints)
tailoredFrontierPlot(object = efficient_frontier)

#Fama French VarCov Estimate

asset.names = colnames(stocks_exret)
asset.names
reg_data <- cbind(stocks_exret, ff5["1992-03-01/2005-06-01",1:5])
#reg_data.df <- as.data.frame(reg_data)
# initialize list object to hold regression objects

reg.list = list()
# loop over all assets and estimate time series regression
for (i in asset.names) {
  reg.df = reg_data[, c(i, "Mkt.RF","SMB","HML","RMW","CMA")]
  si.formula = as.formula(paste(i,"~", "Mkt.RF+SMB+HML+RMW+CMA", sep=" "))
  reg.list[[i]] = lm(si.formula, data=reg.df)
}
summary(reg.list$AMGN_ret_ex)

#Matrix Algebra
returns_matrix=as.matrix(stocks_exret)
factor.mat = as.matrix(ff5["1992-03-01/2005-06-01",1:5])
n.obs = nrow(returns_matrix)
X.mat = cbind(rep(1,n.obs),factor.mat)
colnames(X.mat)[1] = "intercept"
XX.mat = crossprod(X.mat)

# multivariate least squares
G.hat = solve(XX.mat)%*%crossprod(X.mat,returns_matrix)
Mkt.RF.hat = G.hat[2,]
SMB.hat = G.hat[3,]
HML.hat = G.hat[4,]
RMW.hat = G.hat[5,]
CMA.hat = G.hat[6,]
beta.hat = G.hat[2:6,]
E.hat = returns_matrix - X.mat%*%G.hat
diagD.hat = diag(crossprod(E.hat)/(n.obs-6))
# compute R2 values from multivariate regression
sumSquares = apply(returns_matrix, 2, function(x) {sum( (x - mean(x))^2 )})
R.square = 1 - (n.obs-6)*diagD.hat/sumSquares

# compute multiple index model covariance/correlation matrices
cov.mi = t(beta.hat)%*%var(factor.mat)%*%beta.hat + diag(diagD.hat)
#Estimation of sample covariance matrix
cov.sample=var(returns_matrix)
#cor.mi = cov2cor(cov.mi)
#Estimation of mean
mu = matrix(colMeans(returns_matrix), nrow = ncol(returns_matrix), ncol = 1)
e = matrix(1, nrow = nrow(cov.mi), ncol = 1) # unitary column vector e
# compute the column vector with weights of the CER tangency portfolio
w.tan.factor = (solve(cov.mi)%*%(mu))/as.numeric(t(e)%*%(solve(cov.mi)%*%(mu)))
barplot(t(w.tan.factor), horiz=F, main="Weights", col="blue", cex.names = 0.75, las=2)
