# Answers to Q3 

rm(list=ls()) 
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))

listofpackages <- c("tidyverse","ellipse","reshape2","xts","xlsx","readxl",
                    "quantmod")
#installation of "xlsx" requires Java 

for (j in listofpackages){
  if(sum(installed.packages()[, 1] == j) == 0) {
    install.packages(j)
  }
  library(j, character.only = T)
}
install.packages(c("cluster","mvoutlier","pastecs","fPortfolio"),repos="http://cran.r-project.org")
# load required packages
library(cluster)
library(mvoutlier)
library(pastecs)
library(fPortfolio)
library(dygraphs)
#load data 
load('ff5.rda')
load('stocks_ret.rda')
colnames(stocks_ret) <- sub("_ret$", "", colnames(stocks_ret))

data<- (stocks_ret["1992-03-01/2023-10-01"])
companies <- colnames(data)
#getting the data in ts format 
tsdata <- ts(data, start = c(1992, 3), frequency = 12, names = companies)
data01ts <- as.timeSeries(tsdata)

## ------------------------------
#  BACKTESTING   with fPortfolio
## ------------------------------
#getting the data in ts format 
ddown<-drawdowns(data01ts)
ddowndata <- ts(ddown, start = c(1992, 3), frequency = 12, names = companies)
s1 <- window(ddowndata[, "DJI"], start = c(1992, 3), end = c(2023, 10))
dygraph(s1, ylab = "DJI", main = "drawdowns")
drawdownsStats(data01ts[, "DJI"])
## ------------------------------
#  out-of-sample BACKTESTING   
## ------------------------------

Data <- data01ts
Spec <- portfolioSpec()
Constraints <- "LongOnly"
Backtest <- portfolioBacktest()
setWindowsHorizon(Backtest) <- "161m"
equidistWindows(data = Data, backtest = Backtest)

#Specify assets for backtesting
Formula <- DJI ~ AXP +AMGN+AAPL+BA+CAT+CSCO+CVX+
HD+HON+IBM+INTC+JNJ+KO+JPM+ 
MCD+MMM+MRK+MSFT+NKE+PG+TRV+
UNH+VZ+WBA+WMT+DIS

#Optimize rolling portfolios and run backtests
 
btportfolios <- portfolioBacktesting(formula = Formula,
                                     data = data01ts, spec = Spec, constraints = Constraints,
                                     backtest = Backtest, trace = FALSE)


#Weights are rebalanced on a monthly basis
Weights <- round(100 * btportfolios$weights, 2)[1:10, ]
Weights
# change the SmootherLambda to "2400m" to have contstant weights
setSmootherLambda(btportfolios$backtest) <- "1m"
SmoothPortfolios <- portfolioSmoothing(object = btportfolios,trace = FALSE)
smoothWeights <- round(100 * SmoothPortfolios$smoothWeights,2)[1:10, ]
smoothWeights

backtestPlot(SmoothPortfolios, cex = 0.6, font = 1, family = "mono")

netPerformance(SmoothPortfolios)
