# Answers to Q4 

rm(list=ls()) 
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))

listofpackages <- c("tidyverse","ellipse","reshape2","xts","xlsx","readxl",
                    "quantmod")
#installation of "xlsx" requires Java 

for (j in listofpackages){
  if(sum(installed.packages()[, 1] == j) == 0) {
    install.packages(j)
  }
  library(j, character.only = T)
}

#load data 
load('ff5.rda')
load('stocks_ret.rda')
#create excess returns

t1<-nrow(stocks_ret)
names <- colnames(stocks_ret)
for (name in names) {
  return_col_name <- paste0(name,"_ex")
  temporary_column  <- array(data = "na", dim = t1)
  stocks_ret <- merge.xts(stocks_ret, temporary_column) # add last column
  colnames(stocks_ret)[ncol(stocks_ret)] = return_col_name # rename it
}

  for (i in 28:54) {
    stocks_ret[, i] <- stocks_ret[, i-27]-ff5[, 6]
  }
stocks_exret<- (stocks_ret["1992-03-01/2023-10-01",28:53])



#Fama French VarCov Estimate

asset.names = colnames(stocks_exret)
asset.names
reg_data <- cbind(stocks_exret, ff5["1992-03-01/2023-10-01",1:5])
#reg_data.df <- as.data.frame(reg_data)
# initialize list object to hold regression objects

reg.list = list()
# loop over all assets and estimate time series regression
for (i in asset.names) {
  reg.df = reg_data[, c(i, "Mkt.RF","SMB","HML","RMW","CMA")]
  si.formula = as.formula(paste(i,"~", "Mkt.RF+SMB+HML+RMW+CMA", sep=" "))
  reg.list[[i]] = lm(si.formula, data=reg.df)
}
summary(reg.list$AMGN_ret_ex)

#Matrix Algebra
returns_matrix=as.matrix(stocks_exret)
factor.mat = as.matrix(ff5["1992-03-01/2023-10-01",1:5])
n.obs = nrow(returns_matrix)
X.mat = cbind(rep(1,n.obs),factor.mat)
colnames(X.mat)[1] = "intercept"
XX.mat = crossprod(X.mat)

# multivariate least squares
G.hat = solve(XX.mat)%*%crossprod(X.mat,returns_matrix)
Mkt.RF.hat = G.hat[2,]
SMB.hat = G.hat[3,]
HML.hat = G.hat[4,]
RMW.hat = G.hat[5,]
CMA.hat = G.hat[6,]
beta.hat = G.hat[2:6,]
E.hat = returns_matrix - X.mat%*%G.hat
diagD.hat = diag(crossprod(E.hat)/(n.obs-6))
# compute R2 values from multivariate regression
sumSquares = apply(returns_matrix, 2, function(x) {sum( (x - mean(x))^2 )})
R.square = 1 - (n.obs-6)*diagD.hat/sumSquares

# compute multiple index model covariance/correlation matrices
cov.mi = t(beta.hat)%*%var(factor.mat)%*%beta.hat + diag(diagD.hat)
#cor.mi = cov2cor(cov.mi)
#Estimation of mean
mu = matrix(colMeans(returns_matrix), nrow = ncol(returns_matrix), ncol = 1)
e = matrix(1, nrow = nrow(cov.mi), ncol = 1) # unitary column vector e
# compute the column vector with weights of the CER tangency portfolio
w.tan.factor = (solve(cov.mi)%*%(mu))/as.numeric(t(e)%*%(solve(cov.mi)%*%(mu)))
barplot(t(w.tan.factor), horiz=F, main="Weights", col="blue", cex.names = 0.75, las=2)

returnmat<- as.matrix(stocks_ret["1992-03-01/2023-10-01",1:26])
TAN<-returnmat%*%w.tan.factor
datestan <-seq(as.Date("1992-03-01"),length=380, by="months")
TAN_ret<- xts(TAN, order.by=datestan)

rawdata <- merge(stocks_ret$DJI_ret, TAN_ret, join = 'outer') 
rawdata<-as.data.frame(rawdata["1992-03-01/2023-10-01"])

## --------------------
#  MODEL-SIMULATION   
## --------------------

## -----------------------------------
# model specification and estimation 
mod_TAN <- lm(rawdata$TAN_ret ~ 1)
summary(mod_TAN) 
mod_DJI <- lm(rawdata$DJI_ret ~ 1)
summary(mod_DJI)

## -----------------------------------
# parameter calibration and choice of the number of replications in the simulation and of the sample size for simulated data 
vol <- sd(mod_TAN$residuals)
alpha <- mod_TAN$coefficients[[1]]
nrep <- 1000
TT <- nrow(rawdata)
# here I create the containers to be filled with the generated data.
y_bt <- y_mc <- array(1, c(TT, nrep))
x_bt <- x_mc <- array(alpha, c(TT, nrep))

# now, the loop

for (i in 1:nrep){
  u <- rnorm(TT)
  res <- sample(mod_TAN$residuals, replace = T) # this (re)samples from the data
  
  x_mc[, i] <- alpha + vol * u # the Monte Carlo way
  x_bt[, i] <- alpha+res             # the bootstrap way
  
  # now we simply construct and store the bootstrapped and MC cumulative returns
  for (j in 2:TT){
    y_mc[j, i] <- y_mc[j-1, i] * (1 + x_mc[j, i])
    y_bt[j, i] <- y_bt[j-1, i] * (1 + x_bt[j, i])
  }
}

# now we want to construct the series of means and quantiles of the resulting collection of drawn series
for (i in 1:TT){
  # obtaining the means
  rawdata$y_bt_mean[i] <- mean(y_bt[i, ])
  rawdata$x_bt_mean[i] <- mean(x_bt[i, ])
  rawdata$y_mc_mean[i] <- mean(y_mc[i, ])
  rawdata$x_mc_mean[i] <- mean(x_mc[i, ])
  
  # and the quantiles
  rawdata$y_bt_q05[i] <- quantile(y_bt[i, ], 0.05)
  rawdata$x_bt_q05[i] <- quantile(x_bt[i, ], 0.05)
  rawdata$y_mc_q05[i] <- quantile(y_mc[i, ], 0.05)
  rawdata$x_mc_q05[i] <- quantile(x_mc[i, ], 0.05)
  
  rawdata$y_bt_q95[i] <- quantile(y_bt[i, ], 0.95)
 rawdata$x_bt_q95[i] <- quantile(x_bt[i, ], 0.95)
  rawdata$y_mc_q95[i] <- quantile(y_mc[i, ], 0.95)
  rawdata$x_mc_q95[i] <- quantile(x_mc[i, ], 0.95)

}

## --------------------------------------------
# Value at Risk  via Monte Carlo simulation
## --------------------------------------------
s1_mc=x_mc[2,]
hist(s1_mc, breaks = seq(min(s1_mc), max(s1_mc), l = 20+1),prob=TRUE, main = "histogram of monthly returns")
curve(dnorm(x,mean=mean(s1_mc),sd=sd(s1_mc)),col='darkblue',lwd=2,add=TRUE)
VaR_mc <- quantile(s1_mc, 0.05)
VaR_mc

s1_bt=x_bt[2,]
hist(s1_bt, breaks = seq(min(s1_bt), max(s1_bt), l = 20+1),prob=TRUE, main = "histogram of monthly returns")
curve(dnorm(x,mean=mean(s1_bt),sd=sd(s1_bt)),col='darkblue',lwd=2,add=TRUE)
VaR_bt <- quantile(s1_bt, 0.05)
VaR_bt

